 /*
 * file      : src/stream_record.c
 * project   : tnt
 * with      : Gtk-3
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TNT - INDIQUEZ ICI CE QUE FAIT LE PROGRAMME
 * GNU General Public License
 *
 * This file is part of TNT.
 *
 * TNT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 *
 * TNT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TNT.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib.h>

#include <pthread.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>	// PROCESSUS NON-BLOQUANT
#include <poll.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <errno.h>

#include <linux/dvb/frontend.h>
#include <linux/dvb/dmx.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_dvbtv/lib.h"
#endif

#include "global.h"
#include "chan.h"
#include "stream.h"
#include "conv.h"


#define BUFFY (188*20)
#define MAX_LENGTH ( 1024 * 1024 * 20 ) /* record 8 MB */
#define DVR_FILE "/home/cat/DEV/DVB-T/tnt-0.0.3/src/film.avi"


gboolean StreamRecord_switch_to_record( int *p_demux_Video_pid, int *p_demux_Audio_pid, CHANNEL *p_Channel )
{
	struct dmx_pes_filter_params pesFilterParams;

	if( *p_demux_Video_pid < 0 ) {
		if( (*p_demux_Video_pid = open( adapter.demux, O_RDWR | O_NONBLOCK )) < 0 ) {
				perror( "DEMUX-VIDEO-PID DEVICE: " );
				return( FALSE );
		}
	}
	if( *p_demux_Audio_pid < 0 ) {
		if( (*p_demux_Audio_pid = open( adapter.demux, O_RDWR | O_NONBLOCK )) < 0 ) {
				perror( "DEMUX-AUDIO-PID DEVICE: " );
				return( FALSE );
		}
	}
	pesFilterParams.pid      = p_Channel->Video_pid;
	pesFilterParams.input    = DMX_IN_FRONTEND;
	pesFilterParams.output   = DMX_OUT_TS_TAP;
	pesFilterParams.pes_type = DMX_PES_VIDEO;
	pesFilterParams.flags    = DMX_IMMEDIATE_START;
	if( ioctl( *p_demux_Video_pid, DMX_SET_PES_FILTER, &pesFilterParams ) < 0 ) {
			perror( "DEMUX-VIDEO-PID DEVICE" );
			return( FALSE );
	}
	pesFilterParams.pid      = p_Channel->Audio_pid;
	pesFilterParams.input    = DMX_IN_FRONTEND;
	pesFilterParams.output   = DMX_OUT_TS_TAP;
	pesFilterParams.pes_type = DMX_PES_AUDIO;
	pesFilterParams.flags    = DMX_IMMEDIATE_START;
	if( ioctl( *p_demux_Audio_pid, DMX_SET_PES_FILTER, &pesFilterParams ) < 0 ) {
			perror( "DEMUX-AUDIO-PID DEVICE" );
			return( FALSE );
	}
	return( TRUE );
}


// guint vpid	Channel->Video_pid
// guint apid	Channel->Audio_pid
//
static void StreamRecord_thread( void *arg )
{
	CHANNEL		*p_Channel = (CHANNEL *)arg;
	int 		len;
	int 		written;
	guint		buf[ BUFFY ];
	guint 		length = 0;
	struct 		pollfd pfd[1];
	int 		dvr, dvr_out, frontend_fd;
	struct		dvb_frontend_parameters	param;
	int 		demux_Video_pid = -1;
	int 		demux_Audio_pid = -1;

	// open dvr device
	if( (dvr = open( adapter.dvr, O_RDONLY | O_NONBLOCK )) < 0 ){
		perror("DVR DEVICE");
		// return -1;
		pthread_exit(0);
	}

	// set record channel
	if( (frontend_fd = open( adapter.frontend, FE_OPEN_MODE )) < 0 ) {
		close( dvr );
		perror("TUNER DEVICE");
		pthread_exit(0);
	}

	chan_get_param( p_Channel->NameChan, &param );

	ioctl( frontend_fd, FE_SET_FRONTEND, &param );

	// switch video and audio demuxes to dvr
	printf ("Switching dvr on\n");
	if( FALSE == StreamRecord_switch_to_record( &demux_Video_pid, &demux_Audio_pid, p_Channel ) ) {
		close( dvr );
		pthread_exit(0);
	}
	printf("Switching dvr on: Ok\n");
	printf("Recording %2.0f MB of test file in TS format\n", MAX_LENGTH/(1024.0*1024.0));
	length = 0;

	// open output file
	if( (dvr_out = open( DVR_FILE, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP |S_IWGRP | S_IROTH | S_IWOTH)) < 0 ) {
		close( dvr );
		close( frontend_fd );
		close( demux_Video_pid );
		close( demux_Audio_pid );
		perror("Can't open file for dvr test");
		pthread_exit(0);
	}

	pfd[ 0 ].fd     = dvr;
	pfd[ 0 ].events = POLLIN;

	// poll for dvr data and write to file
	while( length < MAX_LENGTH ) {
		if( poll( pfd, 1, 1 )) {
			if( pfd[0].revents & POLLIN ){
				len = read( dvr, buf, BUFFY );
				if( len < 0 ) {
					perror( "recording" );
					break;
				}
				if( len > 0 ) {
					written = 0;
					while( written < len ) {
						written += write( dvr_out, buf, len );
					}
					length += len;
					printf( "written %2.0f MB\r", length/1024./1024. );
				}
			}
		}
	}
	printf("\n");
	close( dvr );
	close( frontend_fd );
	close( dvr_out );
	close( demux_Video_pid );
	close( demux_Audio_pid );
	g_print("\nRecord OK\n");

	// conv_with_args( "ffmpeg", "-y", "-i", "/home/cat/DEV/DVB-T/tnt-0.0.1/src/film.avi", "-vcodec", "copy", "/home/cat/DEV/DVB-T/tnt-0.0.1/src/video.avi", NULL );
	conv_with_args( "ffmpeg", "-y", "-i", "/home/cat/DEV/DVB-T/tnt-0.0.3/src/film.avi", "-vcodec", "copy", "-acodec", "copy", "/home/cat/DEV/DVB-T/tnt-0.0.3/src/video.avi", NULL );
	g_print("Convert Record OK\n");

	pthread_exit(0);
}


void StreamRecord_go( gchar *p_NameChan )
{
	CHANNEL		*Channel = NULL;
	pthread_t	nmr_tid;

	if( TRUE == Stream_scan() ) {
		g_print("New stream OK\n");
		Stream_print();
		Channel = chan_get_struct( p_NameChan );
		pthread_create( &nmr_tid, NULL ,(void *)StreamRecord_thread, (CHANNEL *)Channel );
	}
}








